import json
import os
import pickle
from typing import Any
from typing import Dict
from typing import Optional

import requests


class MizarAPIException(Exception):
    pass


class MizarRequestException(Exception):
    pass


class StrategySizeLimitReached(Exception):
    def __init__(self, signal_size: float, maximum_signal_size_allowed: float):
        message = f"Maximum size of {maximum_signal_size_allowed}MB exceeded. Signal size is {signal_size}MB."
        super().__init__(message)


class Mizar:
    # API_KEY = "mizar-temp-secret-key"
    API_VERSION = "v1"
    API_URL = "{scheme}://{host}/{version}/"

    def __init__(self, api_key=None, api_url=None, scheme="https", host="api.mizar.ai"):
        if not api_key:
            api_key = os.getenv("MIZAR_API_KEY")
            if not api_key:
                raise ValueError("Both api_key and MIZAR_API_KEY are empty")
        self.api_key = api_key
        self.session = self._create_session()
        self.api_url = api_url or self.API_URL
        if scheme not in ("http", "https"):
            raise ValueError("Allowed scheme are http and https")
        self.api_url = self.API_URL.format(
            scheme=scheme, host=host, version=self.API_VERSION
        )
        # check working
        self.ping()

    def _create_session(self):
        session = requests.session()
        session.headers.update(
            {
                "Accept": "application/json",
                "User-Agent": "mizar/python",
                "MIZAR-API-KEY": self.api_key,
            }
        )
        return session

    def _get(self, resource, **kwargs):
        uri = self.api_url + resource
        return self.session.get(uri, **kwargs)

    def _post(self, resource, **kwargs):
        uri = self.api_url + resource
        return self.session.post(url=uri, **kwargs)

    def ping(self):
        resp = self._get("ping")
        if "pong" not in resp.json().get("message", []) or resp.status_code != 200:
            raise MizarAPIException(resp.text)
        return self._handle_response(resp)

    def _handle_response(self, response):
        if response.ok:
            try:
                if response.json().get("data"):
                    return {
                        "status": response.status_code,
                        "url": response.url,
                        "data": response.json().get("data"),
                    }
                elif response.json().get("message"):
                    return {
                        "status": response.status_code,
                        "url": response.url,
                        "message": response.json().get("message"),
                    }
                else:
                    raise MizarRequestException("No message or data")
            except ValueError:
                return MizarRequestException(response.text)
        else:
            raise MizarAPIException(response.json().get("message"))

    def get_quote_assets(self):
        resp = self._get("quote-assets")
        return self._handle_response(resp)

    def get_base_assets(self):
        resp = self._get("base-assets")
        return self._handle_response(resp)

    def get_exchanges(self):
        resp = self._get("exchanges")
        return self._handle_response(resp)

    def get_bar_types(
        self,
        *,
        base_asset: Optional[str] = None,
        quote_asset: Optional[str] = None,
        bar_type: Optional[str] = None,
        bar_subclass: Optional[str] = None,
        exchange: Optional[str] = None,
        **kwargs,
    ):
        """
        Return the bar types

        :param base_asset: Base asset to select (e.g. BTC)
        :type base_asset: str
        :param quote_asset: Quote asset to select (e.g. USDT)
        :type quote_asset: str
        :param bar_type: bar type can be (e.g. tick, dollar, volume, time)
        :type bar_type: str
        :param bar_subclass: bar subclass specify the class of bar type
                             to select (e.g. 1min, 3min, dynamic etc..)
        :type bar_subclass: str
        :param exchange: exchange name
        :type exchange: str
        """
        kwargs.update(
            {
                "base_asset": base_asset,
                "quote_asset": quote_asset,
                "bar_type": bar_type,
                "bar_subclass": bar_subclass,
                "exchange": exchange,
            }
        )

        resp = self._get("bar-types", params=kwargs)
        return self._handle_response(resp)

    def get_bar_data(
        self,
        *,
        base_asset: str,
        quote_asset: str,
        start_timestamp: int = 0,
        limit: int = 500,
        bar_type: str = "time",
        bar_subclass: str = "D",
        exchange: str = "binance",
        **kwargs,
    ):
        """
        Return the bar data for a specific bar type

        :param base_asset: Base asset to select (e.g. BTC)
        :type base_asset: str
        :param quote_asset: Quote asset to select (e.g. USDT)
        :type quote_asset: str
        :param start_timestamp: The timestamp from which collect the bars
        :int start_timestamp: int
        :param limit: max number of bars to fetch (maximum allowed limit is 500)
        :type limit: int
        :param exchange: exchange name
        :type exchange: str
        :param bar_type: bar type can be (e.g. tick, dollar, volume, time)
        :type bar_type: str
        :param bar_subclass: bar subclass specify the class of bar type
                             to select (e.g. 1min, 3min, dynamic etc..)
        :type bar_subclass: str
        :return:

        [
            {
                'bar_type_id': 1,
                'base_asset_buy_volume': 105736.42,
                'base_asset_id': 184,
                'base_asset_sell_volume': 5325.2,
                'base_asset_volume': 111061.61999999998,
                'close': 0.27,
                'exchange_id': 1,
                'first_timestamp': 1523937753168,
                'first_trade_id': 0,
                'high': 0.27,
                'id': 1,
                'last_timestamp': 1523937778928,
                'last_trade_id': 14,
                'low': 0.25551,
                'num_buy_ticks': 13,
                'num_sell_ticks': 2,
                'num_ticks': 15,
                'open': 0.25551,
                'quote_asset_buy_volume': 27380.973451500002,
                'quote_asset_id': 4,
                'quote_asset_sell_volume': 1362.102,
                'quote_asset_volume': 28743.0754515,
                'time': '2018-04-17 04:02:00+00:00'
            },
            {
                etc
            },
            etc
        ]
        """
        kwargs.update(
            {
                "base_asset": base_asset,
                "quote_asset": quote_asset,
                "start_timestamp": start_timestamp,
                "limit": limit,
                "exchange": exchange,
                "bar_type": bar_type,
                "bar_subclass": bar_subclass,
            }
        )
        resp = self._get("bar", params=kwargs)
        return self._handle_response(resp)

    def save_strategy_signal(
        self,
        strategy,
        strategy_signal_info: Dict[str, Any],
    ):
        # TODO: later stage check if strategy-info entries match with
        #  strategy properties
        import codecs

        MAXIMUM_SIGNAL_SIZE_ALLOWED = 200

        encoded_strategy = codecs.encode(pickle.dumps(strategy), "base64")

        encoded_strategy_size = len(encoded_strategy) / 1e6

        if encoded_strategy_size > MAXIMUM_SIGNAL_SIZE_ALLOWED:
            raise StrategySizeLimitReached(
                encoded_strategy_size, MAXIMUM_SIGNAL_SIZE_ALLOWED
            )
        resp = self._post(
            "save-strategy",
            # do not change the way the data is posted to mizar
            # the strategy info are dumped in a json string because
            # data does not support nested dictionaries. The values in data
            # haveto be not nested
            data={
                "strategy-info": json.dumps(strategy_signal_info),
                # protocol is set to 0 otherwise we lose info when
                # the pickle object is transformed to string
                "strategy": encoded_strategy.decode(),
            },
        )

        return self._handle_response(resp)



import os

import pytest
import requests_mock

from mizar.api import Mizar
from mizar.api import MizarAPIException


@pytest.fixture()
def client():
    client = Mizar("api_key")
    return client


def test_missing_key():
    os.environ["MIZAR_API_KEY"] = ""
    with pytest.raises(ValueError):
        Mizar()


def test_not_allowed_key():
    with pytest.raises(ValueError):
        Mizar("api_key", scheme="not_allowed_scheme")


def test_key_from_environ():
    os.environ["MIZAR_API_KEY"] = "test"
    Mizar()


def test_ping_api_exception(client):
    with pytest.raises(MizarAPIException):
        with requests_mock.mock() as m:
            m.get(
                "https://api.mizar.ai/v1/ping",
                json={"message": "API exception"},
                status_code=400,
            )
            client.ping()


def test_ping_working(client):
    with requests_mock.mock() as m:
        m.get(
            "https://api.mizar.ai/v1/ping",
            json={"message": "pong"},
            status_code=200,
        )
        client.ping()



import json
import os

import pandas as pd

from mizar.api import Mizar


class MizarStudio:
    def __init__(self, mizar: Mizar, path: str):
        self.mizar = mizar
        self.path = path
        if path[-1] != "/":
            path = f"{path}/"

        if not os.path.isdir(path):
            os.makedirs(path)

    def get_bar_df(
        self,
        base_asset: str,
        quote_asset: str,
        start_timestamp: int = 0,
        bar_type: str = "time",
        bar_subclass: str = "D",
        exchange: str = "binance",
    ) -> pd.DataFrame:
        """
        :param base_asset: Base asset to select (e.g. BTC)
        :type base_asset: str
        :param quote_asset: Quote asset to select (e.g. USDT)
        :type quote_asset: str
        :param start_timestamp: The timestamp from which collect the bars
        :param exchange: exchange name
        :type exchange: str
        :param bar_type: bar type can be (e.g. tick, dollar, volume, time)
        :type bar_type: str
        :param bar_subclass: bar subclass specify the class of bar type
                             to select (e.g. 1min, 3min, dynamic etc..)
        :type bar_subclass: str
        :return: bar dataframe
        :rtype: pd.DataFrame
        """
        dir = f"{self.path}bar/{exchange}/{base_asset}{quote_asset}/{bar_type}"

        if os.path.isfile(f"{dir}/{bar_subclass}_bar_data.csv"):
            bars_df = pd.read_csv(f"{dir}/{bar_subclass}_bar_data.csv")
            timestamp = bars_df["time"].max()

        else:
            os.makedirs(f"{dir}/")
            bars_df = pd.DataFrame()
            timestamp = start_timestamp

        previous_size = -1

        while bars_df.shape[0] != previous_size:
            previous_size = bars_df.shape[0]

            response = self.mizar.get_bar_data(
                base_asset=base_asset.upper(),
                quote_asset=quote_asset.upper(),
                start_timestamp=timestamp,
                bar_type=bar_type,
                bar_subclass=bar_subclass,
                exchange=exchange,
            )

            bars_df = pd.concat(
                [bars_df, pd.DataFrame(**json.loads(response["data"]))], axis=0
            )
            bars_df.drop_duplicates(
                inplace=True, ignore_index=True, subset=["first_trade_id"]
            )
            timestamp = bars_df["time"].max()
        bars_df.sort_values(by="time", inplace=True)
        bars_df.to_csv(f"{dir}/{bar_subclass}_bar_data.csv", index=False)
        return bars_df



import os
import shutil
from functools import wraps

import pytest

from mizar.api import Mizar
from mizar.studio import MizarStudio


@pytest.fixture()
def mizar_studio():
    mizar = Mizar()
    return MizarStudio(mizar, path="./mizar_studio_test")


def delete_test_folder(f):
    @wraps(f)
    def _delete_test_folder(*args, **kwargs):

        try:
            return f(*args, **kwargs)
        finally:
            shutil.rmtree("./mizar_studio_test")

    return _delete_test_folder


@delete_test_folder
def test_mizar_studio_folder_creation(mizar_studio):
    assert os.path.isdir("./mizar_studio_test")



from pathlib import Path

from setuptools import find_packages
from setuptools import setup

__version__ = "0.0.2"

ROOT_DIR = Path(".")

with open("requirements.txt") as f:
    requirements = f.readlines()

with open(str(ROOT_DIR / "README.md")) as readme:
    long_description = readme.read()

setup(
    name="mizar",
    version=__version__,
    author="MizarAI",
    author_email="info@mizar.ai",
    description="Official API for the MizarAI platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MizarAI/mizar",
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=requirements,
    project_urls={
        "Bug Tracker": "https://github.com/MizarAI/mizar/issues",
    },
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Topic :: Office/Business",
        "Topic :: System :: Networking",
        "Topic :: Communications :: Chat",
        "Intended Audience :: Developers",
    ],
)
